package at.gv.egiz.moazs.scheme;

import at.gv.zustellung.app2mzs.xsd.Mzs2AppPortType;
import at.gv.zustellung.msg.xsd.DeliveryAnswerType;

import javax.xml.bind.JAXBElement;
import java.util.Optional;
import java.util.concurrent.CompletableFuture;

/**
 * Represents responses to DeliveryRequests that were received from the msg service.
 * @param <T> The type of the response.
 */
public abstract class MsgResponse <T> {

    protected String id;

    protected MsgResponse(){
        this.id = "";
    }

    public static String createResponseId(String appDeliveryID, String idSuffix) {
        return appDeliveryID + idSuffix;
    }

    public String getResponseID() {
        return id;
    }
    public String getRootElementLocalPart() {return getResponseAsJAXBElement().getName().getLocalPart();}

    public abstract DeliveryAnswerType getAnswer();
    public abstract T getResponse();
    public abstract JAXBElement<T> getResponseAsJAXBElement();
    public abstract String getAppDeliveryID();
    public abstract String getZSDeliveryID();

    /**
     * Create an error response that is based on the current response.
     * @param text describes the error.
     * @param code contains an error code.
     * @return Deep Copy of response but of "error" type
     */
    public abstract MsgResponse<T> generateError(String text, String code);
    public abstract CompletableFuture<Void> sendToMzsClient(Msg2MzsConverter converter, Optional<byte[]> signedStatus, Mzs2AppPortType client);

}
